#!/usr/bin/python
import re
import sys, os

fileName = "./Testfile"
f = None

# since we go through the file linewise, we shouldn't have any newline where we don't want it
reComment = r'\s*//.*\n'

# \s is equal to [ \n\r\t] and so on
reEmpty = r'\s*\n'

# the whitespaces can be tricky
# just take defines, which don't have variable names in them
# it works with them too, but it is unneccessary work
#reDefine = r'\s*`define (?P<defName>\w*)[ ]+(?P<defContent>[^`]?[\w.\-+ ]+)\n'
reDefine = r'\s*`define (?P<defName>\w*)[ ]+(?P<defContent>((^`\w+)|[\w.+\-*/])+)\n'

# We have to differ between simple substitutions and functions with 1/2/3 variables
reSubstitute =r'\s*(`define )?[\w.+\-*/ \(\)]*(?P<toSub>`[^(define)](\w_\-)*)[\w.+\-*/ ]*\n'
#reSubstitute =r'\s*(`define)*[\w ]*(?P<toSub>`[^define]\w*)[ \w]*\n'

# fucntions with 1 parameter
#reDefineFunc1 = r'\s*`define (?P<funName>\w*)\((?P<varName>\w*)\) (?P<funBody>\(((?P<alreadyDefined>`\w*)|(?P=varName)|[.0-9a-zA-Z+\-*//\(\)])*\))\n'
reDefineFunc1 = r'\s*`define (?P<funName>\w*)\((?P<varName>\w*)\) (?P<funBody>\(((?P=varName)|[.0-9a-zA-Z+\-*//\(\)])*\))\n'

# build the dictionary base on what our RE will find
myDict = None

def init():
    global f, myDict
    print("init started")
    
    # initialize the global values
    myDict = dict()

    # check if file exists...
    if (not os.path.exists(fileName)):
        print("File not found!")
        exit(1)
    f = open(fileName)

#    print(re.compile(reSubstitute).search('`define VT(temp) (`P_K*temp/`P_Q)').groupdict())
#    print(re.fullmatch(reSubstitute, '`define VT(temp) (`P_K*temp/`P_Q)'))

def finish():
    print("finish started")
    global f, myDict
    print(myDict)
    f.close()

def processStep(toProcess):
    print("processStep was started")
    global myDict
    iDidSomething = False

    for element in range(len(toProcess)):
        # pass comments, and other stuff we won't parse
        if (re.fullmatch(reComment, toProcess[element])):
            pass
        if(re.fullmatch(reEmpty, toProcess[element])):
            pass

        if (re.fullmatch(reSubstitute, toProcess[element])):
            print(re.compile(reSubstitute).search(toProcess[element]).group('toSub'))
            print("I can substitute something!!!")
            print(toProcess[element])

        if (re.fullmatch(reDefineFunc1, toProcess[element])):
            print("Function definition found")
            (k, v,v2) = re.compile(reDefineFunc1).search(toProcess[element]).group('funName', 'funBody', 'varName')
            myDict.update({k:(v,v2)})
            toProcess[element] = "//" + toProcess[element]
            iDidSomething = True
            pass

        # if we find a definition we will add them to our dict
        if (re.fullmatch(reDefine, toProcess[element])):
            print("Definition found")
            (k, v) = re.compile(reDefine).search(toProcess[element]).group('defName', 'defContent')
            myDict.update({k:v})
            toProcess[element] = "//" + toProcess[element]
            iDidSomething = True
            pass

    
            
    # tell process if something was done here
    return iDidSomething


# read every line, process until nothing was processed the last time
def process():
    print("process started")
    global f

    # put everything into toProcess
    toProcess = []
    workWasDone = True
    buf = f.readline()
    while(buf):
        toProcess.append(buf)
        buf=f.readline()

    while(workWasDone):
        workWasDone = False
        workWasDone = processStep(toProcess)

    # check the output for debugging purposes
    out = open('out.va', 'w')
    for elem in toProcess:
        out.write(elem)
    out.close()

if __name__ == "__main__":
    print("main started")
    init()
    process()
    finish()
